/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file mser.cpp
/// @brief Example program for QVision library. ???????????
///
/// To use the program, simply compile, and execute the binary:
///
///	./mser --Video=<path or url to video>
///
/// @author PARP Research Group. University of Murcia, Spain.
///

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <QDebug>

#include <qvcore/qvapplication.h>
#include <qvmplayercamera/qvmplayercamera.h>
#include <qvgui/qvisioninterface.h>

#include <qvdta/qvdta.h>
#include <qvdta/qvcontour.h>
#include <qvdta/qvcomponenttree.h>

#define	ABS(X)	((X>=0)?X:-X)

class QVMSER
	{
	public:
		QVMSER(QPoint s, uChar t): seed(s), threshold(t) {};

		QPoint seed;
		uChar threshold;
	};

class MyWorker: public QVWorker
	{
	public:
		MyWorker(QString name): QVWorker(name)
			{
			addProperty<int>("Delta", inputFlag, 1, "MSER parameter, as seen in the paper.", 1, 128);
			addProperty< QVImage<uChar,1> >("Input image", inputFlag|outputFlag);
			addProperty< QVImage<uChar,3> >("MSER regions", outputFlag);
			}

		void worker()
			{
			//////////////////
			// Read parameters
			const int delta = getPropertyValue<int>("Delta");
			const QVImage<uChar> image = getPropertyValue< QVImage<uChar,1> >("Input image");
			const uInt rows = image.getRows(), cols = image.getCols();
			timeFlag("Read parameters");
		
			/////////////////
			// Prune low areas from image, with component tree
			QVImage<uChar> sinAreasMin = image;
			qvdta::QVComponentTree componentTree(image);
			timeFlag("Call to getComponentTree for low areas");

			/////////////////
			// Tree transversing
			QList<QVMSER> MSERList;

			uInt histogram[256];
			double q_function[256];

			//std::cout << "start" << std::endl;
			for (uInt node = 0; node < componentTree.getNumNodes(); node++)
				{

				int firstThres = componentTree.firstThreshold(node), lastThres = componentTree.lastThreshold(node);

				if (lastThres - firstThres - 2*delta - 2 < 0)
					continue;

				// We obtain histogram
				for (int threshold = firstThres, lastHistogramValue; threshold <= lastThres; threshold++)
						{
						uInt area = componentTree.area(node)[threshold];
						if(area != 0)
							lastHistogramValue = area;
						histogram[threshold] = lastHistogramValue;
						}

				// Calculate 'q' function (from the paper)
				for (int threshold = firstThres + delta; threshold <= lastThres - delta; threshold++)
					{
					q_function[threshold] = (double)(histogram[threshold + delta] - histogram[threshold - delta]) / histogram[threshold];
					}

				bool found = false;
				// Get local minima of the function, and append them to MSER list
				for (int threshold = firstThres + delta + 1; threshold <= lastThres - delta - 1; threshold++)
					{
					if ( (q_function[threshold + 1] > q_function[threshold]) && (q_function[threshold - 1] > q_function[threshold]) )
						{
						MSERList.append(QVMSER(QPoint(componentTree.seedX(node), componentTree.seedY(node)), threshold));
						found = true;
						}
					}

				// Imprimimos las tablas
				if (found)
					{
					std::cout << "node " << node << "; seed = (" << componentTree.seedX(node) << "," << componentTree.seedY(node) << ")"<< std::endl;
					std::cout << "\thistogram" << std::endl;
					for (int threshold = firstThres; threshold <= lastThres; threshold++)
						{
						std::cout << "\t\t" << threshold << "\t" << histogram[threshold] << std::endl;
						}

					std::cout << "\tq function (delta = " << delta << ")" << std::endl;
					for (int threshold = firstThres + delta; threshold <= lastThres - delta; threshold++)
						{
						std::cout << "\t\t" << threshold << "\t" << q_function[threshold]
							<< "\t(histogram[" << threshold + delta << "] - histogram[" << threshold - delta << "]) / histogram[" << threshold << "]"  << std::endl;
						}
					}
				}
			timeFlag("MSER");

			////////////////////
			// Draw MSER's
			QVImage<uChar> notImage(cols, rows);
			for (uInt col = 0; col < cols; col++)
				for (uInt row = 0; row < rows; row++)
					notImage(col, row) = 255 - image(col, row);


			QList< qvdta::QVPolyline > polylineMSERList;

			for (uInt msers = 0; msers < MSERList.size(); msers++)
				{
				qvdta::QVPolyline polylineMSER = qvdta::getBorderContourThreshold(notImage, MSERList.at(msers).seed,  255 - MSERList.at(msers).threshold);
				polylineMSERList.append (polylineMSER);
				}

			QVImage<uChar,3> mserImage = image;

 			uChar redPixel[3] = { 255, 0, 0 };

			draw(mserImage, polylineMSERList, redPixel);

			/////////////////
			// Publish resulting images
			setPropertyValue< QVImage<uChar,3> >("MSER regions",mserImage);
			timeFlag("Publish resulting images");
			}
	};

int main(int argc, char *argv[])
	{
	QVApplication app(argc, argv,
		
		"Example program for QVision library. Gets component tree from images."

		);
	
	QVMPlayerCamera camera("Video");
	MyWorker worker("MSER Worker");
	QVisionInterface interface;

	camera.link(&worker,"Input image");
	interface.setVisible(&worker,"Input image");
	interface.setVisible(&worker,"MSER regions");

	return app.exec();
	}


	
