/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvcomponenttree.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qvdta/qvdta.h>

#define	 NULL_NODE	256*256*256

namespace qvdta
{
/// @class QVComponentTree qvdta/qvcomponenttree.h
/// @brief This class can be used to create the component tree from a gray-scale image.
///
/// A level set in a gray-scale image is a connected set of points (pixels) with a gray-scale level
/// above a given threshold. Component trees are built upon the inclusion relation of the level sets in
/// an image.
///
/// They are a representation useful to obtain non-flat connected component filters, that may preserve
/// connected components in the image, satisfying a given criterion. These trees are used in computer
/// vision for task like image segmentation, filtering, scale and rotation invariant image feature
/// extraction (like MSER, see [<a href="#matas">Matas</a>]), amongst other.
///
/// For a wider explanation about component trees, you can read paper [<a href="#rjones">Jones</a>].
/// The former implementation of component trees is very efficient, so that it can be used with regular
/// sized images (320x240 to 640x480, depending on the computer speed) in real-time programs.
/// It is based in the algorithm described in the paper [<a href="#najman">Najman</a>], which is supossed
/// to obtain the component tree in quasi-linear time.
///
/// There's a main difference between this implementation and that described in the paper. While the latter
/// constructs a node for each gray-level in a level set, this implementation compacts every gray-level
/// included in a same level set, in one node only. It will only be stored for every gray-level in the same
/// node it's histogram area; thus, every non-leaf node in QVision's component tree will have more than one
/// descendant, making smaller trees.
///
/// The function @ref rootNode returns the index of the parent root of the tree. You can check if that node (or any node) is a
/// leaf node, with the function @ref numChilds, which returns the number of descendant nodes that has a given node from the
/// tree. For any non-leaf node, you can transverse the list of it's descendant nodes using first the function @ref firstChild
/// to get the index of the first of it's child nodes, and then using the function @ref nextSibling to get the next child in the
/// list, for that child to the last of the list.
///
/// For every node index, you can get a seed point of the level set it represents (that is, generally a given point of the set)
/// using functions @ref seedX and @ref seedY, to get horizontal and vertical coordinates of the point. Functions
/// @ref firstThreshold, @ref lastThreshold, and @ref area can be used to obtain histogram information about gray-scale values
/// for the level set of the node.
///
/// REFERENCES:<ul>
/// <li><a name="rjones"><i>Component trees for image filtering and segmentation</i></a>. Ronald Jones.</li>
/// <li><a name="najman"><i>Building the component tree in quasi-linear time</i></a>. L. Najman and M. Couprie.</li>
/// <li><a name="matas"><i>Robust Wide Baseline Stereo from Maximally Stable Extremal Regions</i>. J. Matas et al.</li>
/// @ingroup qvdta
class QVComponentTree
	{
	public:

		/// @brief Constructs a component tree set, from a gray-scale image.
		///
		/// Root node will embrace the whole image, and will descend to the lower gray-scale level valued areas.
		///
		/// @param image image to create from the component tree.
		QVComponentTree(const QVImage<uChar,1> &image, bool inverseTree= false);

		~QVComponentTree();

		bool isInverseTree()	const		{ return inverseTree; }
		uInt rootNode()	const			{ return rootNodeID; }
		uInt seedX(uInt node) const		{ return nodes[node].seedX; }
		uInt seedY(uInt node) const		{ return nodes[node].seedY; }
		uChar firstThreshold(uInt node) const	{ return nodes[node].firstThreshold; }
		uChar lastThreshold(uInt node) const	{ return nodes[node].lastThreshold; }
		uInt numChilds(uInt node) const		{ return nodes[node].numChilds; }
		uInt firstChild(uInt node) const	{ return nodes[node].child; }
		uInt nextSibling(uInt node) const	{ return nodes[node].sibling; }
		uInt *area(uInt node) const		{ return nodes[node].area; }
		bool validNode(uInt node) const 	{ return nodes[node].inited; }

		uInt & rootNode()			{ return rootNodeID; }
		uInt & seedX(uInt node) 		{ return nodes[node].seedX; }
		uInt & seedY(uInt node) 		{ return nodes[node].seedY; }
		uChar & firstThreshold(uInt node)	{ return nodes[node].firstThreshold; }
		uChar & lastThreshold(uInt node) 	{ return nodes[node].lastThreshold; }
		uInt & numChilds(uInt node) 		{ return nodes[node].numChilds; }
		uInt & firstChild(uInt node)	 	{ return nodes[node].child; }
		uInt & nextSibling(uInt node) 		{ return nodes[node].sibling; }
		uInt *area(uInt node) 			{ return nodes[node].area; }
		bool & validNode(uInt node) 		{ return nodes[node].inited; }

		uInt getNumNodes() const	{ return numNodes; }
		uInt getTotalPoints() const	{ return totalPoints; }
		uInt getLeafNodes() const	{ return leafNodes; }

	private:
		void getComponentTree(const QVImage<uChar> &image);
		uInt numNodes, freePoints, totalPoints, leafNodes, rootNodeID, maxNodes;
		bool inverseTree;

		uInt newNode(uInt SeedX, uInt SeedY, uChar Threshold)
			{
			uInt newNodeID = this->numNodes++;
			seedX(newNodeID) = SeedX;
			seedY(newNodeID) = SeedY;
			firstThreshold(newNodeID) = lastThreshold(newNodeID) = Threshold;
			firstChild(newNodeID) =	 nextSibling(newNodeID) = NULL_NODE;
			numChilds(newNodeID) = 0;
			area(newNodeID)[Threshold] = 0;
			validNode(newNodeID) = false;

			return newNodeID;
			}

		void addChild(uInt ParentNodeID, uInt ChildNodeID)
			{
			nextSibling(ChildNodeID) = firstChild(ParentNodeID);
			firstChild(ParentNodeID) = ChildNodeID;
			numChilds(ParentNodeID)++;
			}

		class QVComponentTreeNode
			{
			public:
				uInt seedX, seedY;
				uInt child, sibling, numChilds;
				uChar firstThreshold, lastThreshold;
				uInt area[256];
				bool inited;
			} *nodes;
	};

//uInt debug_print_componentTree(QVComponentTree &componentTree);
void FilterComponentTreeSmallRegions(QVImage<uChar> &image, QVComponentTree &componentTree, uInt area);

}
