/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvcontour.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVCONTOURS_H
#define QVCONTOURS_H

#include <qvdta/qvpolyline.h>

namespace qvdta
{
/// @brief Obtains the border contour of a connected set of pixels in an image, given a membership condition.
///
/// A contour is a sequence of pixels from an image, were every two contiguous pixels in the sequence are 8-neigbourhood in
/// the image.
///
/// This function obtains the contour that depicts the border set of pixels in an image, for a
/// <a href="http://en.wikipedia.org/wiki/Connected_set">connected set</a> of pixels in it, given a membership condition.
///
/// For this border contour extractor function, the membership condition will be that pixels will be considered to be members of
/// a connected if and only if their gray-scale value is equal or greater than a given threshold value.
/// 
/// The function implements an automata, that will receive an image and a starting point in its arguments, and will look for 
/// the closest border pixel to that point.
///
/// That border pixel will have a gray-level value equal or higher than a threshol value, given also in the arguments of the
/// function, and will transverse the border of the set, appending every point that it finds in it in a @ref QVPolyline object,
/// that will be the returning value of the function.
///
/// Border contours obtained with this function can be either inner borders (corresponding to the border of an empty space inside
/// the connected set) or the outer border of the connected set. The programer can tell wether a given border is inner or outer
/// by the boolean value @ref QVPolyline::direction, which will store <i>TRUE</i> for an outer contour, and <i>FALSE</i> for an
/// inner contour.
///
/// @param image image from were to obtain the poly-line.
/// @param point any point inside the connected set.
/// @param threshold threshold to consider pixels inside or outside the connected set.
/// @returns a QVPolyline that follows the border set of the connected set.
/// @see getContoursThreshold
QVPolyline getBorderContourThreshold(const QVImage<uChar> &image, const QPoint point, const uChar threshold);

/// @brief Obtains a list of the border contours of the connected sets in an image, according to a membership condition.
///
/// This function gets the border contours for all of the <a href="http://en.wikipedia.org/wiki/Connected_set">connected sets</a>
/// of pixels in an image, using the function @ref getContourThreshold, and returns them in a list of polylines
/// (QList<QVPolyline>).
///
/// Thus, it considers every point inside a connected set, if it has a gray-level equal or higher than a given value.
///
/// @param image image from were to obtain the poly-lines.
/// @param threshold threshold to consider pixels inside or outside the connected sets.
/// @returns a QList of QVPolyline's that follows the border sets of the connected sets in the image.
/// @see getContourThreshold
//void getContoursThreshold(const QVImage <uChar> &image, const uChar threshold, QList<QVPolyline> &result);
QList<QVPolyline> getBorderContoursThreshold(const QVImage <uChar> &image, const uChar threshold);
}

#endif

